"""
Subpackage for genrating plots

The modules in this subpackage typically define functions that plot something on matplotlib axes passed as first argument, based on the content of some astropy tables.
"""
__all__ = []

import utils
import scatter
import hist
import hexbin
import contour
import bin
import mcbin
import figures
import exp


