#!/usr/bin/env python

from distutils.core import setup

setup(
	name='tenbilac',
	version='1.0dev',
	packages=['tenbilac'],
	description='Inverse regression network',
	long_description=open('README.md').read(),
	author='Malte Tewes',
	license='GPLv3',
	url='http://github.com/mtewes/tenbilac'
)

